#ifndef _CCF_HPP_
 #define _CCF_HPP_

/** 
 *	@file Variable.hpp implementation and definition of class Variable.
 *	@author Kento EMOTO
 */
#ifndef _VARIABLE_HPP_
#define _VARIABLE_HPP_

/** independent parameter.
 *	@author Kento EMOTO
*/
class Variable {
public:
	double val;					/**< value of the parameter */
	/**
	 * an id of this parameter.
	 * if the id is zero, the parameter indicates zero. (is treated as zero)
	 */
	int id;
	/**
	 * constructor: initialized as zero
	 */
	Variable(){
		val = 0;
		id = 0;
	}
	/**
	 * constructor: id is initialized to i
	 * @param i id of this parameter
	 */
	Variable(int i){
		val = 0;
		id = i;
	}
	/**
	 * set id of this parameter
	 * @param i new id of this parameter
	 */
	void setId(int i){
		id = i;
	}
	/**
	 * true if this parameter is zero.
	 */
	bool isZero(){
		return id==0;
	}
	/**
	 *	output this parameter like 't' + id
	 */
	friend ostream &operator<<(ostream& stream, Variable v){
		if(v.id!=0){
			stream << 't' << v.id;
		} else {
			stream << 0;
		}
		return stream;
	}
};

#endif
/** 
 *	@file VariableMatrix.hpp implementation and definition of class VariableMatrix.
 *	@author Kento EMOTO
 */
#ifndef _VARIABLEMATRIX_HPP_
#define _VARIABLEMATRIX_HPP_

/** matrix of independent parameters
 * @see Variable
 *	@author Kento EMOTO
 */
class VariableMatrix {
public:
	Variable *ele;				/**< elements */
	int col;					/**< the number of columns */
	int row;					/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	VariableMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	VariableMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** destructor */
	~VariableMatrix()
	{
		freeMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new Variable[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	Variable get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat Variable to be put to (i, j)
	 */
	void put(int i, int j, Variable dat){
		ele[i*col + j] = dat;
	}
};
#endif
/** 
 *	@file RealModularNumber.hpp implementation and definition of class RealModularNumber.
 *	@author Kento EMOTO
 */
#ifndef _MODULERNUMBER_HPP_
#define _MODULERNUMBER_HPP_

#include <iostream>
#include <iomanip>

using namespace std;
#include <stdlib.h>
#include <time.h>

extern int thePrime;		/**< prime used by RealModularNumber */

extern double eps;

void setEps(double e);
double getEps();

int setPrime(int p);

extern int exp_id;
/**
 *	an exception thrown when the prime is not luckey.
 */
class ThePrimeIsNotLuckeyException {
public:
	int e;
	ThePrimeIsNotLuckeyException(){e = exp_id++;}
	/**
	 *	stream output operator(<<).
	 */
	friend ostream &operator<<(ostream &stream, ThePrimeIsNotLuckeyException &exp){
		stream << "ThePrimeIsNotLuckeyException(" << exp.e << ")";
		return stream;
	}
};

/**
 *	number for moduler calculus
 */
class RealModularNumber {
public:
	/**
	 *	number in Z_p.
	 *	num should be normalized as 0 <= num < thePrime 
	 *	or -thePrime/2 <= num < thePrime/2
	 */
	int num;
	/**
	 *	floating point representation.
	 */
	 double val;

	/**
	 *	destructor.
	 */
	~RealModularNumber(){
	}
	/**
	 *	constructor: 0.
	 */
	RealModularNumber(){
		num = 0;
		val = 0;
	}
	/**
	 *	constructor: Integer n.
	 */
	RealModularNumber(int n){
		num = n % thePrime;
		val = n;
	}
	/**
	 *	constructor: Rational n/d.
	 */
	RealModularNumber(int n, int d){
		int inv = getInverse(d, thePrime);
		num = (inv * (long long)n) % thePrime;
		val = n/(double)d;
	}
	/**
	 *	copy constructor.
	 */
	RealModularNumber(const RealModularNumber &mn){
		num = mn.num;
		val = mn.val;
	}
	/**
	 *	addition operator(+).
	 */
	RealModularNumber operator+(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num + (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val + mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}
	/**
	 *	subtraction operator(-).
	 */
	RealModularNumber operator-(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num - (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val - mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	multiplicatoin operator(*).
	 */
	RealModularNumber operator*(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num * (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val * mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	division operator(/).
	 */
	RealModularNumber operator/(const RealModularNumber &mn){
		RealModularNumber mn2;
		int inv = getInverse(mn.num, thePrime);
		mn2.num = (num * (long long)inv) % thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val / mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	negation operator(-).
	 */
	RealModularNumber operator-(){
		RealModularNumber mn2;
		mn2.num = (- num) % thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = - val;
		return mn2;
	}

	/**
	 *	substitution operator(=).
	 */
	RealModularNumber &operator=(const RealModularNumber &mn){
		num = mn.num;
		val = mn.val;
		return *this;
	}
	/**
	 *	stream output operator(<<).
	 */
	friend ostream &operator<<(ostream &stream, RealModularNumber &mn){
		stream << "[";
		stream << mn.num;
		stream << "]";
		stream << "(" << mn.val << ")";
		return stream;
	}
	/**
	 *	return true if this is zero on all fields('Z_p's).
	 */
	bool isZero(){
		if(num!=0) return false;
		val = 0;
		return true;
	}
	/**
	 *	return true if this is not zero on all fields('Z_p's).
	 */
	bool isNotZero(){
		if(num==0) return false;
		return true;
	}

protected:
	/**
	 *	get inverse of n in Z_p.
	 *	i.e. integer m satisfying n * m + p * l = 1
	 */
	int getInverse(int n, int p){
		if(n==0){
			cout << "divide by zero? in RealModularNumber::getInverse(p = " << p << ")\n";
		}
		int sn = 1;
		if(n < 0){
			sn = -1;
			n = -n;
		}
		int a0 = 0;
		int a1 = 1;
		int b0 = 1;
		int b1 = 0;
		int r0 = p;
		int r1 = n;
		int d;
		int tmp;
		while(r1!=0){
			d = r0 / r1;
			tmp = r0 % r1;
			r0 = r1;
			r1 = tmp;
			tmp = a0 - d * a1;
			a0 = a1;
			a1 = tmp;
			tmp = b0 - d * b1;
			b0 = b1;
			b1 = tmp;
		}
		
		return (a0 * sn);
	}
};

#endif
/** 
 *	@file RealModularMatrix.hpp implementation and definition of class RealModularMatrix.
 *	@author Kento EMOTO
 */
#ifndef _REALMODULERMATRIX_HPP_
#define _REALMODULERMATRIX_HPP_

/** matrix of multi-precision rational numbers.
 * (i, j) - element of this matrix is stored in ele[i*col+j] (0 <= i < row, 0<= j < col).
 * @see RealModularNumber
 *	@author Kento EMOTO
 */
class RealModularMatrix {
public:
	RealModularNumber *ele;			/**< elements (i, j) -> [i*col+j] */
	int col;						/**< the number of columns */
	int row;						/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	RealModularMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	RealModularMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** destructor */
	~RealModularMatrix()
	{
		freeMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new RealModularNumber[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	RealModularNumber &get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat RealModularNumber to be put to (i, j)
	 */
	void put(int i, int j, RealModularNumber &dat){
		ele[i*col + j] = dat;
	}
	/**
	 *	duplicate from another matrix.
	 *	@param rm another matrix which this matrix duplicates
	 */
	void copyFrom(const RealModularMatrix &rm){
		init(rm.row, rm.col);
		int rc = col*row;
		for(int i = 0; i < rc; i++){
			ele[i] = rm.ele[i];
		}
	}
	/** 
	 * make this an identity matrix of rxr.
	 *	@param r row of the matrix
	 */
	void setIdentity(int r){
		init(r, r);
		RealModularNumber rnOne(1);
		for(int i = 0; i < r; i++){
			put(i,i,rnOne);
		}
	}
};
#endif
/** 
 *	@file RationalNumber.hpp implementation and definition of class RationalNumber.
 *	@author Kento EMOTO
 */
#ifndef _RATIONAL_NUMBER_HPP_
#define _RATIONAL_NUMBER_HPP_

#include <iostream>
#include <iomanip>

using namespace std;

/**
 *	rational number.
 *	represented by (num, den), i.e. the value = num/den
 *	@author Kento EMOTO
 */
class RationalNumber {
public:
	int den;			/**< denominator */
	int num;			/**< numerator */
	
	/** constructor: initialized as zero */
	RationalNumber(){
		den = 1;
		num = 0;
	}
	/** constructor: initialized as integer value n */
	RationalNumber(int n){
		den = 1;
		num = n;
	}
	/** constructor: initialized as rational value n/d */
	RationalNumber(int n, int d){
		den = d;
		num = n;
	}
	/** operator taking RationalNumber rn(RHS) and returning this + rn */
	RationalNumber operator+(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this - rn */
	RationalNumber operator-(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this * rn */
	RationalNumber operator*(const RationalNumber &rn);
	/** operator taking RationalNumber rn(RHS) and returning this / rn */
	RationalNumber operator/(const RationalNumber &rn);
	/** operator returning - this */
	RationalNumber operator-(){
		return RationalNumber(-num, den);
	}
	void canonicalize();
	
	/** output to stream */
	friend ostream &operator<<(ostream &stream, RationalNumber rn){
		stream << rn.num << "/" << rn.den;
		return stream;
	}
	/** true if this value is zero */
	bool isZero() { return num==0; }
	/** return double value of this */
	double doubleValue() { return num/(double)den; }
	/** return int value of this denominator */
	int intDenValue() { return den; }
	/** return int value of this numerator */
	int intNumValue() { return num; }
protected:
	int gcd(int x, int y);
};

#endif
/** 
 *	@file RationalMatrix.hpp implementation and definition of class RationalMatrix.
 *	@author Kento EMOTO
 */
#ifndef _RATIONALMATRIX_HPP_
#define _RATIONALMATRIX_HPP_

/** matrix of multi-precision rational numbers.
 * (i, j) - element of this matrix is stored in ele[i*col+j] (0 <= i < row, 0<= j < col).
 * @see RationalNumber
 *	@author Kento EMOTO
 */
class RationalMatrix {
public:
	RationalNumber *ele;			/**< elements (i, j) -> [i*col+j] */
	int col;						/**< the number of columns */
	int row;						/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	RationalMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	RationalMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** destructor */
	~RationalMatrix()
	{
		freeMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new RationalNumber[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	RationalNumber &get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat RealNumber to be put to (i, j)
	 */
	void put(int i, int j, RationalNumber dat){
		ele[i*col + j] = dat;
	}
	/**
	 *	duplicate from another matrix.
	 *	@param rm another real matrix which this matrix duplicates
	 */
	void copyFrom(const RationalMatrix &rm){
		init(rm.row, rm.col);
		int rc = col*row;
		for(int i = 0; i < rc; i++){
			ele[i] = rm.ele[i];
		}
	}
	/** 
	 * make this an identity matrix of rxr.
	 *	@param r row of the matrix
	 */
	void setIdentity(int r){
		init(r, r);
		RationalNumber rnOne(1);
		for(int i = 0; i < r; i++){
			put(i,i,rnOne);
		}
	}
};
#endif
/** 
 *	@file RealNumber.hpp implementation and definition of class RealNumber.
 *	@author Kento EMOTO
 */
#ifndef _REALNUMBER_HPP_
#define _REALNUMBER_HPP_

/**
 * wrapper class of double.
 *	@author Kento EMOTO
 */
class RealNumber {
public:
	double d;			/**< value */
	/** constructor: initialized as zero */
	RealNumber(){
		d = 0;
	}
	/** constructor: initialized as integer value n */
	RealNumber(int n){
		d = n;
	}
	/** constructor: initialized as double value dd */
	RealNumber(double dd){
		d = dd;
	}
	/** operator taking RealNumber rn(RHS) and returning this + rn */
	RealNumber operator+(const RealNumber &rn){
		RealNumber r(d+rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this - rn */
	RealNumber operator-(const RealNumber &rn){
		RealNumber r(d-rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this * rn */
	RealNumber operator*(const RealNumber &rn){
		RealNumber r(d*rn.d);
		return r;
	}
	/** operator taking RealNumber rn(RHS) and returning this / rn */
	RealNumber operator/(const RealNumber &rn){
		RealNumber r(d/rn.d);
		return r;
	}
	/** operator returning - this */
	RealNumber operator-(){
		RealNumber r(-d);
		return r;
	}
	/** output to stream */
	friend ostream &operator<<(ostream &stream, RealNumber rn){
		stream << rn.d;
		return stream;
	}
	/** true if this value is zero */
	bool isZero() { return d==0; }
	/** return this value */
	double doubleValue() {return d; }
};
#endif
/** 
 *	@file RealMatrix.hpp implementation and definition of class RealMatrix.
 *	@author Kento EMOTO
 */
#ifndef _REALMATRIX_HPP_
#define _REALMATRIX_HPP_


/** matrix of real numbers.
 * (i, j) - element of this matrix is stored in ele[i*col+j] (0 <= i < row, 0<= j < col).
 * @see RealNumber
 *	@author Kento EMOTO
 */
class RealMatrix {
public:
	RealNumber *ele;			/**< elements */
	int col;					/**< the number of columns */
	int row;					/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	RealMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	RealMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** destructor */
	~RealMatrix()
	{
		freeMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new RealNumber[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	RealNumber get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat RealNumber to be put to (i, j)
	 */
	void put(int i, int j, RealNumber dat){
		ele[i*col + j] = dat;
	}
	/**
	 *	duplicate from another matrix.
	 *	@param rm another real matrix which this matrix duplicates
	 */
	void copyFrom(const RealMatrix &rm){
		init(rm.row, rm.col);
		int rc = col*row;
		for(int i = 0; i < rc; i++){
			ele[i] = rm.ele[i];
		}
	}
	/** 
	 * make this an identity matrix of rxr.
	 *	@param r row of the matrix
	 */
	void setIdentity(int r){
		init(r, r);
		RealNumber one(1);
		for(int i = 0; i < r; i++){
			put(i,i,one);
		}
	}
};
#endif
/** 
 *	@file MixedMatrix.hpp definition of class MixedMatrix.
 *	@author Kento EMOTO
 */
#ifndef _MIXEDMATRIX_HPP_
#define _MIXEDMATRIX_HPP_

#include <iostream>
#include <iomanip>




using namespace std;

/**
 *	a class for mixed matrix.
 *	This holds a mixed matrix and calculates CCF of it. The result is held by this.
 *  The algorith for CCF is based on chapter 2 of "d (): U\ƃASY I (ߑȊw, 1992.)"
 *
 *  CCF = CCF_Q + CCF_T = Pr * S * (Q + T) * Pc
 *
 * sample code (reading from standard in):
 * <pre>
 *	MixedMatrix mm;                      // create an instance
 *	mm.readFromStream(cin);             // read a mixed amtrix form standard in
 *	mm.writeToStream(cout);                // print the matrix to standard out
 *	mm.step1();                            // STEP1(initialize)
 *	mm.step2(true);                        // STEP2(calculate CCF)
 *	cout << "rank = " << mm.rank << "\n";  // rank of matrix
 *	mm.writeToStreamCCF(cout);             // print CCF 
 * </pre>
 * sample code (manually put elements into Q and T from stream):
 * <pre>
 *	istream in;                          // some input stream
 *	MixedMatrix mm;                      // create an instance
 *	mm.init(rq,rt,c);                    // set the size
 *	for(int i = 0; i < rowQ; i++){       // elements of Q (integer)
 *		for(int j = 0; j < col; j++){
 *			int n;
 *			in >> n;
 *			RationalNumber rn(n);
 *			matQ.put(i, j, rn);       // put an element
 *		}
 *	}
 *	for(int i = 0; i < rowT; i++){       // elements of T (id of params)
 *		for(int j = 0; j < col; j++){
 *			int n;
 *			in >> n;
 *			Variable var(n);          // n = 0 means zero
 *			matT.put(i, j, var);
 *		}
 *	}
 * </pre>
 *
 * define _RATIONAL_ to use multi-precision rational numbers, which provides you precise results. However, it takes much time to calculate CCF. Moreover, it needs GMP(GNU MP), which is a library of multi-precision numbers, to compile source files.
 * @see SCD
 * @see Graph
 *	@author Kento EMOTO
 */
class MixedMatrix {
public:
	int rowQ;							/**< the number of rows of Q */
	int rowT;							/**< the number of rows of T */
	int col;							/**< the number of columns of Q (T) */
	RationalMatrix matQ;				/**< matrix Q */
	VariableMatrix matT;				/**< matrix T */


	RealMatrix matS;					/**< transformation matrix S */
	/** column permutation.
	 * Pc(i, Pc[i]) = 1, otherwise 0 for column permutation matrix Pc
	 */
	int *Pc;						
	/** row permutation.
	 * Pr(Pr[i], i) = 1, otherwise 0 for row permutation matrix Pr
	 */
	int *Pr;
	/** column block structure. 
	 *  Cs[0] and Rs[0] are the number of columns and rows of holizontal tail.
	 *  Cs[blocknum+1] and Rs[blocknum+1] are the number of columns and rows of vertical tail.
	 *  Cs[i] and Rs[i] are the number of columns and rows of ith non-singular block (1 <= i <= blocknum).
	 *
	 * @see MixedMatrix#blocknum
	 * @see MixedMatrix#Rs
	 */
	int *Cs;
	/** row block structure. 
	 * @see MixedMatrix#Cs
	 */						
	int *Rs;						
	RealMatrix matCCFQ;			/**< CCF Q part */
	VariableMatrix matCCFT;			/**< CCF T part */

	int blocknum;				/**< the number of non-singular blocks of CCF*/
	int rank;					/**< rank of the matrix */

	MixedMatrix();
	void init(int rq, int rt, int c);
	void readFromStream(istream &in);
	void writeToStream(ostream &out);
	void step1();
	void step2(bool CCFflag);
	~MixedMatrix();
	void printall();
	void printvertex(int v);
	void writeMatrixToStream(ostream &out, RationalMatrix &mat);
	void writeMatrixToStream(ostream &out, RealMatrix &mat);
	void writeToStreamCCF(ostream &out);

	/** return the threshold.
	 * @see MixedMatrix#setThreshold(double t)
	 */
	double getThreshold() { return doubleThreshold; }
	/** set the threshold. 
	 * Each real number obtained by elimination is treated as zero 
	 * if its absolute value is smaller than the threshild set by this method.
	 * Setting this value to zero disables this feature.
	 * @see MixedMatrix#getThreshold()
	 */
	void setThreshold(double t) { doubleThreshold = t; }
protected:
	const static int inI = 0x01;	/**< flag indicating an element of set I */
	const static int inJ = 0x02;		/**< of set J */
	const static int inSminus = 0x04;	/**< of set S- */
	const static int inSplusT = 0x08;	/**< of set S+T */
	const static int inSplusQ = 0x10;	/**< of set S+Q */
	const static int inG0 = 0x20;		/**< of set G0 */
	const static int inGinf = 0x40;		/**< of set Ginf */
	const static int theBTC = 0x01;		/**< edge of BT BC */
	const static int theBP = 0x02;		/**< edge of BP */
	const static int theM = 0x04;		/**< edge of M */
	int flags[3];						/**< theBTC, theBP, theM */
	RealMatrix matP;					/**< eliminated matrix P */
	int *base;							/**< bases of P (making rank) */

	/** information of edges of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTChead;
	/** adjacency list of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTCadjlist;
	/** the top of adjacency list of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTCvfirst;

	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPhead;
	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPadjlist;
	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPvfirst;

	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Mhead;
	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Madjlist;
	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Mvfirst;
	int *vattr;						/**< attributes of vertices */
	int *pathL;						/**< edges of path seached in step2 */
	int *pathLinfo;					/**< information of found path */
	int *v_label;					/**< visited flag */
	int *v_info;					/**< kinds of edges used */
	int *v_que1;					/**< que for WFS */
	int **heads;					/**< BTC, BP, M head */
	int **adjlists;					/**< BTC, BP, M adjlist */
	int **vfirsts;					/**< BTC, BP, M vfirst */
	/** threshold for real number.
	 * @see MixedMatrix#setThreshold(double t)
	 */
	double doubleThreshold;			
	int vnum;					/**< the number of edges of vertices of graph*/
	int BTCnum;					/**< the number of edges of BT and BC */
	int BCnum;					/**< the number of edges of BC */
	int BTnum;					/**< maximum of the number of edges of BT */
	int BPnum;					/**< the number of edges of BP */
	int BPmax;					/**< maximum of the number of edges of BP */
	int Mnum;					/**< the number of edges of M  */
	int Mmax;					/**< maximum of the number of edges of M */
	int pathLnum;				/**< the length of a found path */
	void step4();
	void allocMemory();
	void freeMemory();
	void freeMemoryCCF();
	void allocMemoryCCF();
	void adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n);
	void print_adjlist(int *vfirst, int *head, int *adjlist, int n);
	bool path_search();
	
	/** eliminated matrix P of RealModular Number */
	RealModularMatrix matPM;
	RealModularMatrix matSM;
	/** index of prime numbers used */
	int idxOfPrimesUsed;

	int *base_back;							/**< backup of base */
	int *Mhead_back;						/**< backup of Mhead */

	
	bool reEliminationRealModular();
	void sortBasesByPc(int *bases, int *tmp, int s, int e);
	void duplicateRealModularMatrixFromRationalMatrix(RealModularMatrix &mm, RationalMatrix &rm);
	void step1_forM();
	void step2_updatingM();
	void step3M();
	void allocMemoryForRealModular();
	void freeMemoryForRealModular();
	void writeMatrixToStream(ostream &out,RealModularMatrix &mat);

	bool reEliminationRealModularUpperTriangularByPc();
	void duplicateRealMatrixFromRationalMatrix(RealMatrix &mm, RationalMatrix &rm);

	double *BPweight;			/**< weight of BP edges */
	double *BTCweight;			/**< weight of BTC edges */
	double *weight;				/**< weight of vertices */
	int *v_lastvs;				/**< vertex visited last */
	int *v_layer;				/**< layer of vertices */

	int search_one(int i, int layer, int *que, int *n, double w);



};


#endif
#endif

